@@ -1,12 +1,72 @@
-from core import C
-from singleton import S
-from operations import AssocOp
-from cache import cacheit
-from numbers import ilcm, igcd
+from __future__ import print_function, division
 
 from collections import defaultdict
 
-class Add(AssocOp):
+from .basic import C, Basic
+from .compatibility import cmp_to_key, reduce, is_sequence
+from .logic import _fuzzy_group, fuzzy_or, fuzzy_not
+from .singleton import S
+from .operations import AssocOp
+from .cache import cacheit
+from .numbers import ilcm, igcd
+from .expr import Expr
+
+
+# Key for sorting commutative args in canonical order
+_args_sortkey = cmp_to_key(Basic.compare)
+def _addsort(args):
+    # in-place sorting of args
+    args.sort(key=_args_sortkey)
+
+
+def _unevaluated_Add(*args):
+    """Return a well-formed unevaluated Add: Numbers are collected and
+    put in slot 0 and args are sorted. Use this when args have changed
+    but you still want to return an unevaluated Add.
+
+    Examples
+    ========
+
+    >>> from sympy.core.add import _unevaluated_Add as uAdd
+    >>> from sympy import S, Add
+    >>> from sympy.abc import x, y
+    >>> a = uAdd(*[S(1.0), x, S(2)])
+    >>> a.args[0]
+    3.00000000000000
+    >>> a.args[1]
+    x
+
+    Beyond the Number being in slot 0, there is no other assurance of
+    order for the arguments since they are hash sorted. So, for testing
+    purposes, output produced by this in some other function can only
+    be tested against the output of this function or as one of several
+    options:
+
+    >>> opts = (Add(x, y, evaluated=False), Add(y, x, evaluated=False))
+    >>> a = uAdd(x, y)
+    >>> assert a in opts and a == uAdd(x, y)
+
+    """
+    args = list(args)
+    newargs = []
+    co = S.Zero
+    while args:
+        a = args.pop()
+        if a.is_Add:
+            # this will keep nesting from building up
+            # so that x + (x + 1) -> x + x + 1 (3 args)
+            args.extend(a.args)
+        elif a.is_Number:
+            co += a
+        else:
+            newargs.append(a)
+    _addsort(newargs)
+    if co:
+        newargs.insert(0, co)
+    return Add._from_args(newargs)
+
+
+class Add(Expr, AssocOp):
 
     __slots__ = []
 
@@ -38,29 +98,8 @@ class Add(AssocOp):
             a, b = seq
             if b.is_Rational:
                 a, b = b, a
-            assert a
             if a.is_Rational:
                 if b.is_Mul:
-                    # if it's an unevaluated 2-arg, expand it
-                    c, t = b.as_coeff_Mul()
-                    if t.is_Add:
-                        h, t = t.as_coeff_Add()
-                        bargs = [c*ti for ti in Add.make_args(t)]
-                        bargs.sort(key=hash)
-                        ch = c*h
-                        if ch:
-                            bargs.insert(0, ch)
-                        b = Add._from_args(bargs)
-                if b.is_Add:
-                    bargs = list(b.args)
-                    if bargs[0].is_Number:
-                        bargs[0] += a
-                        if not bargs[0]:
-                            bargs.pop(0)
-                    else:
-                        bargs.insert(0, a)
-                    rv = bargs, [], None
-                elif b.is_Mul:
                     rv = [a, b], [], None
             if rv:
                 if all(s.is_commutative for s in rv[0]):
@@ -70,7 +109,7 @@ class Add(AssocOp):
         terms = {}      # term -> coeff
                         # e.g. x**2 -> 5   for ... + 5*x**2 + ...
 
-        coeff = S.Zero  # standalone term (Number or zoo will always be in slot 0)
+        coeff = S.Zero  # coefficient (Number or zoo) to always be in slot 0
                         # e.g. 3 + ...
         order_factors = []
 
@@ -84,12 +123,14 @@ class Add(AssocOp):
                         break
                 if o is None:
                     continue
-                order_factors = [o]+[o1 for o1 in order_factors if not o.contains(o1)]
+                order_factors = [o] + [
+                    o1 for o1 in order_factors if not o.contains(o1)]
                 continue
 
             # 3 or NaN
             elif o.is_Number:
-                if o is S.NaN or coeff is S.ComplexInfinity and o.is_bounded is False:
+                if (o is S.NaN or coeff is S.ComplexInfinity and
+                        o.is_finite is False):
                     # we know for sure the result will be nan
                     return [S.NaN], [], None
                 if coeff.is_Number:
@@ -100,7 +141,7 @@ class Add(AssocOp):
                 continue
 
             elif o is S.ComplexInfinity:
-                if coeff.is_bounded is False:
+                if coeff.is_finite is False:
                     # we know for sure the result will be nan
                     return [S.NaN], [], None
                 coeff = S.ComplexInfinity
@@ -116,18 +157,11 @@ class Add(AssocOp):
             elif o.is_Mul:
                 c, s = o.as_coeff_Mul()
 
-                # 3*...
-                # unevaluated 2-arg Mul, but we always unfold it so
-                # it can combine with other terms (just like is done
-                # with the Pow below)
-                if c.is_Number and s.is_Add:
-                    seq.extend([c*a for a in s.args])
-                    continue
-
             # check for unevaluated Pow, e.g. 2**3 or 2**(-1/2)
             elif o.is_Pow:
                 b, e = o.as_base_exp()
-                if b.is_Number and (e.is_Integer or (e.is_Rational and e.is_negative)):
+                if b.is_Number and (e.is_Integer or
+                                   (e.is_Rational and e.is_negative)):
                     seq.append(b**e)
                     continue
                 c, s = S.One, o
@@ -137,26 +171,26 @@ class Add(AssocOp):
                 c = S.One
                 s = o
 
-
             # now we have:
             # o = c*s, where
             #
             # c is a Number
             # s is an expression with number factor extracted
-
             # let's collect terms with the same s, so e.g.
             # 2*x**2 + 3*x**2  ->  5*x**2
             if s in terms:
                 terms[s] += c
+                if terms[s] is S.NaN:
+                    # we know for sure the result will be nan
+                    return [S.NaN], [], None
             else:
                 terms[s] = c
 
-
         # now let's construct new args:
         # [2*x**2, x**3, 7*x**4, pi, ...]
         newseq = []
         noncommutative = False
-        for s,c in terms.items():
+        for s, c in terms.items():
             # 0*s
             if c is S.Zero:
                 continue
@@ -170,34 +204,34 @@ class Add(AssocOp):
                     # so we can simply put c in slot0 and go the fast way.
                     cs = s._new_rawargs(*((c,) + s.args))
                     newseq.append(cs)
-
+                elif s.is_Add:
+                    # we just re-create the unevaluated Mul
+                    newseq.append(Mul(c, s, evaluate=False))
                 else:
                     # alternatively we have to call all Mul's machinery (slow)
-                    newseq.append(Mul(c,s))
+                    newseq.append(Mul(c, s))
 
             noncommutative = noncommutative or not s.is_commutative
 
         # oo, -oo
         if coeff is S.Infinity:
-            newseq = [f for f in newseq if not (f.is_nonnegative or f.is_real and
-                                                (f.is_bounded or
-                                                 f.is_finite or
-                                                 f.is_infinitesimal))]
+            newseq = [f for f in newseq if not
+                      (f.is_nonnegative or f.is_real and f.is_finite)]
+
         elif coeff is S.NegativeInfinity:
-            newseq = [f for f in newseq if not (f.is_nonpositive or f.is_real and
-                                                (f.is_bounded or
-                                                 f.is_finite or
-                                                 f.is_infinitesimal))]
+            newseq = [f for f in newseq if not
+                      (f.is_nonpositive or f.is_real and f.is_finite)]
+
         if coeff is S.ComplexInfinity:
             # zoo might be
-            #   unbounded_real + bounded_im
-            #   bounded_real + unbounded_im
-            #   unbounded_real + unbounded_im
-            # addition of a bounded real or imaginary number won't be able to
-            # change the zoo nature; if unbounded a NaN condition could result if
-            # the unbounded symbol had sign opposite of the unbounded portion of zoo,
-            # e.g. unbounded_real - unbounded_real
-            newseq = [c for c in newseq if not (c.is_bounded and
+            #   infinite_real + finite_im
+            #   finite_real + infinite_im
+            #   infinite_real + infinite_im
+            # addition of a finite real or imaginary number won't be able to
+            # change the zoo nature; adding an infinite qualtity would result
+            # in a NaN condition if it had sign opposite of the infinite
+            # portion of zoo, e.g., infinite_real - infinite_real.
+            newseq = [c for c in newseq if not (c.is_finite and
                                                 c.is_real is not None)]
 
         # process O(x)
@@ -219,13 +253,8 @@ class Add(AssocOp):
                     coeff = S.Zero
                     break
 
-
         # order args canonically
-        # Currently we sort things using hashes, as it is quite fast. A better
-        # solution is not to sort things at all - but this needs some more
-        # fixing. NOTE: this is used in primitive and Mul.flattten, too, so if
-        # it changes here it should be changed there.
-        newseq.sort(key=hash)
+        _addsort(newseq)
 
         # current code expects coeff to be first
         if coeff is not S.Zero:
@@ -264,7 +293,7 @@ class Add(AssocOp):
         for ai in a.args:
             c, m = ai.as_coeff_Mul()
             d[m].append(c)
-        for k, v in d.iteritems():
+        for k, v in d.items():
             if len(v) == 1:
                 d[k] = v[0]
             else:
@@ -282,7 +311,7 @@ class Add(AssocOp):
         Examples
         ========
 
-        >>> from sympy.abc import x, y
+        >>> from sympy.abc import x
         >>> (7 + 3*x).as_coeff_add()
         (7, (3*x,))
         >>> (7*x).as_coeff_add()
@@ -316,23 +345,25 @@ class Add(AssocOp):
 
     # Note, we intentionally do not implement Add.as_coeff_mul().  Rather, we
     # let Expr.as_coeff_mul() just always return (S.One, self) for an Add.  See
-    # issue 2425.
+    # issue 5524.
 
+    @cacheit
     def _eval_derivative(self, s):
-        return Add(*[f.diff(s) for f in self.args])
+        return self.func(*[a.diff(s) for a in self.args])
 
     def _eval_nseries(self, x, n, logx):
         terms = [t.nseries(x, n=n, logx=logx) for t in self.args]
-        return Add(*terms)
+        return self.func(*terms)
 
     def _matches_simple(self, expr, repl_dict):
         # handle (w+3).matches('x+5') -> {w: x+2}
         coeff, terms = self.as_coeff_add()
-        if len(terms)==1:
+        if len(terms) == 1:
             return terms[0].matches(expr - coeff, repl_dict)
         return
 
-    matches = AssocOp._matches_commutative
+    def matches(self, expr, repl_dict={}, old=False):
+        return AssocOp._matches_commutative(self, expr, repl_dict, old)
 
     @staticmethod
     def _combine_inverse(lhs, rhs):
@@ -388,19 +419,20 @@ class Add(AssocOp):
         # check for quick exit
         if len(nd) == 1:
             d, n = nd.popitem()
-            return Add(*[_keep_coeff(ncon, ni) for ni in n]), _keep_coeff(dcon, d)
+            return self.func(
+                *[_keep_coeff(ncon, ni) for ni in n]), _keep_coeff(dcon, d)
 
         # sum up the terms having a common denominator
-        for d, n in nd.iteritems():
+        for d, n in nd.items():
             if len(n) == 1:
                 nd[d] = n[0]
             else:
-                nd[d] = Add(*n)
+                nd[d] = self.func(*n)
 
         # assemble single numerator and denominator
-        denoms, numers = [list(i) for i in zip(*nd.iteritems())]
-        n, d = Add(*[Mul(*(denoms[:i]+[numers[i]]+denoms[i+1:]))
-                   for i in xrange(len(numers))]), Mul(*denoms)
+        denoms, numers = [list(i) for i in zip(*iter(nd.items()))]
+        n, d = self.func(*[Mul(*(denoms[:i] + [numers[i]] + denoms[i + 1:]))
+                   for i in range(len(numers))]), Mul(*denoms)
 
         return _keep_coeff(ncon, n), _keep_coeff(dcon, d)
 
@@ -410,17 +442,41 @@ class Add(AssocOp):
     def _eval_is_rational_function(self, syms):
         return all(term._eval_is_rational_function(syms) for term in self.args)
 
+    def _eval_is_algebraic_expr(self, syms):
+        return all(term._eval_is_algebraic_expr(syms) for term in self.args)
+
     # assumption methods
-    _eval_is_real = lambda self: self._eval_template_is_attr('is_real', when_multiple=None)
-    _eval_is_antihermitian = lambda self: self._eval_template_is_attr('is_antihermitian', when_multiple=None)
-    _eval_is_bounded = lambda self: self._eval_template_is_attr('is_bounded', when_multiple=None)
-    _eval_is_hermitian = lambda self: self._eval_template_is_attr('is_hermitian', when_multiple=None)
-    _eval_is_imaginary = lambda self: self._eval_template_is_attr('is_imaginary', when_multiple=None)
-    _eval_is_integer = lambda self: self._eval_template_is_attr('is_integer', when_multiple=None)
-    _eval_is_commutative = lambda self: self._eval_template_is_attr('is_commutative')
+    _eval_is_real = lambda self: _fuzzy_group(
+        (a.is_real for a in self.args), quick_exit=True)
+    _eval_is_complex = lambda self: _fuzzy_group(
+        (a.is_complex for a in self.args), quick_exit=True)
+    _eval_is_antihermitian = lambda self: _fuzzy_group(
+        (a.is_antihermitian for a in self.args), quick_exit=True)
+    _eval_is_finite = lambda self: _fuzzy_group(
+        (a.is_finite for a in self.args), quick_exit=True)
+    _eval_is_hermitian = lambda self: _fuzzy_group(
+        (a.is_hermitian for a in self.args), quick_exit=True)
+    _eval_is_integer = lambda self: _fuzzy_group(
+        (a.is_integer for a in self.args), quick_exit=True)
+    _eval_is_rational = lambda self: _fuzzy_group(
+        (a.is_rational for a in self.args), quick_exit=True)
+    _eval_is_algebraic = lambda self: _fuzzy_group(
+        (a.is_algebraic for a in self.args), quick_exit=True)
+    _eval_is_commutative = lambda self: _fuzzy_group(
+        a.is_commutative for a in self.args)
+
+    def _eval_is_imaginary(self):
+        rv = _fuzzy_group(a.is_imaginary for a in self.args)
+        if rv is False:
+            return rv
+        iargs = [a*S.ImaginaryUnit for a in self.args]
+        r = _fuzzy_group(a.is_real for a in iargs)
+        if r:
+            s = self.func(*iargs, evaluate=False)
+            return fuzzy_not(s.is_zero)
 
     def _eval_is_odd(self):
-        l = [f for f in self.args if not (f.is_even==True)]
+        l = [f for f in self.args if not (f.is_even is True)]
         if not l:
             return False
         if l[0].is_odd:
@@ -443,17 +499,17 @@ class Add(AssocOp):
         if self.is_number:
             return super(Add, self)._eval_is_positive()
         pos = nonneg = nonpos = unknown_sign = False
-        unbounded = set()
+        saw_INF = set()
         args = [a for a in self.args if not a.is_zero]
         if not args:
             return False
         for a in args:
             ispos = a.is_positive
-            ubound = a.is_unbounded
-            if ubound:
-                unbounded.add(ispos)
-                if len(unbounded) > 1:
-                    return None
+            infinite = a.is_infinite
+            if infinite:
+                saw_INF.add(fuzzy_or((ispos, a.is_nonnegative)))
+                if True in saw_INF and False in saw_INF:
+                    return
             if ispos:
                 pos = True
                 continue
@@ -463,26 +519,17 @@ class Add(AssocOp):
             elif a.is_nonpositive:
                 nonpos = True
                 continue
-            elif a.is_zero:
-                continue
 
-            if ubound is None:
-                # sign is unknown; if we don't know the boundedness
-                # we're done: we don't know. That is technically true,
-                # but the only option is that we have something like
-                # oo - oo which is NaN and it really doesn't matter
-                # what sign we apply to that because it (when finally
-                # computed) will trump any sign. So instead of returning
-                # None, we pass.
-                pass
-            else:
-                return None
+            if infinite is None:
+                return
             unknown_sign = True
 
-        if unbounded:
-            return unbounded.pop()
+        if saw_INF:
+            if len(saw_INF) > 1:
+                return
+            return saw_INF.pop()
         elif unknown_sign:
-            return None
+            return
         elif not nonpos and not nonneg and pos:
             return True
         elif not nonpos and pos:
@@ -494,17 +541,17 @@ class Add(AssocOp):
         if self.is_number:
             return super(Add, self)._eval_is_negative()
         neg = nonpos = nonneg = unknown_sign = False
-        unbounded = set()
+        saw_INF = set()
         args = [a for a in self.args if not a.is_zero]
         if not args:
             return False
         for a in args:
             isneg = a.is_negative
-            ubound = a.is_unbounded
-            if ubound:
-                unbounded.add(isneg)
-                if len(unbounded) > 1:
-                    return None
+            infinite = a.is_infinite
+            if infinite:
+                saw_INF.add(fuzzy_or((isneg, a.is_nonpositive)))
+                if True in saw_INF and False in saw_INF:
+                    return
             if isneg:
                 neg = True
                 continue
@@ -514,24 +561,17 @@ class Add(AssocOp):
             elif a.is_nonnegative:
                 nonneg = True
                 continue
-            elif a.is_zero:
-                continue
 
-            if ubound is None:
-                # sign is unknown; if we don't know the boundedness
-                # we're done: we don't know. That is technically true,
-                # but the only option is that we have something like
-                # oo - oo which is NaN and it really doesn't matter
-                # what sign we apply to that because it (when finally
-                # computed) will trump any sign. So instead of returning
-                # None, we pass.
-                pass
+            if infinite is None:
+                return
             unknown_sign = True
 
-        if unbounded:
-            return unbounded.pop()
+        if saw_INF:
+            if len(saw_INF) > 1:
+                return
+            return saw_INF.pop()
         elif unknown_sign:
-            return None
+            return
         elif not nonneg and not nonpos and neg:
             return True
         elif not nonneg and neg:
@@ -547,26 +587,30 @@ class Add(AssocOp):
         coeff_old, terms_old = old.as_coeff_Add()
 
         if coeff_self.is_Rational and coeff_old.is_Rational:
-            if terms_self == terms_old:                       # (2 + a).subs( 3 + a, y) -> -1 + y
-                return Add( new, coeff_self, -coeff_old)
-            if terms_self == -terms_old:                      # (2 + a).subs(-3 - a, y) -> -1 - y
-                return Add(-new, coeff_self,  coeff_old)
+            if terms_self == terms_old:   # (2 + a).subs( 3 + a, y) -> -1 + y
+                return self.func(new, coeff_self, -coeff_old)
+            if terms_self == -terms_old:  # (2 + a).subs(-3 - a, y) -> -1 - y
+                return self.func(-new, coeff_self, coeff_old)
 
-            args_old, args_self = Add.make_args(terms_old), Add.make_args(terms_self)
-            if len(args_old) < len(args_self):    # (a+b+c+d).subs(b+c,x) -> a+x+d
+        if coeff_self.is_Rational and coeff_old.is_Rational \
+                or coeff_self == coeff_old:
+            args_old, args_self = self.func.make_args(
+                terms_old), self.func.make_args(terms_self)
+            if len(args_old) < len(args_self):  # (a+b+c).subs(b+c,x) -> a+x
                 self_set = set(args_self)
                 old_set = set(args_old)
 
                 if old_set < self_set:
                     ret_set = self_set - old_set
-                    return Add(new, coeff_self, -coeff_old,
+                    return self.func(new, coeff_self, -coeff_old,
                                *[s._subs(old, new) for s in ret_set])
 
-                args_old = Add.make_args(-terms_old)     # (a+b+c+d).subs(-b-c,x) -> a-x+d
+                args_old = self.func.make_args(
+                    -terms_old)     # (a+b+c+d).subs(-b-c,x) -> a-x+d
                 old_set = set(args_old)
                 if old_set < self_set:
                     ret_set = self_set - old_set
-                    return Add(-new, coeff_self, coeff_old,
+                    return self.func(-new, coeff_self, coeff_old,
                                *[s._subs(old, new) for s in ret_set])
 
     def removeO(self):
@@ -579,9 +623,9 @@ class Add(AssocOp):
             return self._new_rawargs(*args)
 
     @cacheit
-    def extract_leading_order(self, *symbols):
+    def extract_leading_order(self, symbols, point=None):
         """
-        Returns the leading term and it's order.
+        Returns the leading term and its order.
 
         Examples
         ========
@@ -596,7 +640,10 @@ class Add(AssocOp):
 
         """
         lst = []
-        seq = [(f, C.Order(f, *symbols)) for f in self.args]
+        symbols = list(symbols if is_sequence(symbols) else [symbols])
+        if not point:
+            point = [0]*len(symbols)
+        seq = [(f, C.Order(f, *zip(symbols, point))) for f in self.args]
         for ef, of in seq:
             for e, o in lst:
                 if o.contains(of) and o != of:
@@ -614,7 +661,7 @@ class Add(AssocOp):
 
     def as_real_imag(self, deep=True, **hints):
         """
-        returns a tuple represeting a complex numbers
+        returns a tuple representing a complex number
 
         Examples
         ========
@@ -622,6 +669,10 @@ class Add(AssocOp):
         >>> from sympy import I
         >>> (7 + 9*I).as_real_imag()
         (7, 9)
+        >>> ((1 + I)/(1 - I)).as_real_imag()
+        (0, 1)
+        >>> ((1 + 2*I)*(1 + 3*I)).as_real_imag()
+        (-5, 5)
         """
         sargs, terms = self.args, []
         re_part, im_part = [], []
@@ -640,38 +691,40 @@ class Add(AssocOp):
         if not self.is_Add:
             return self.as_leading_term(x)
 
-        unbounded = [t for t in self.args if t.is_unbounded]
-        if unbounded:
-            return Add._from_args(unbounded)
+        infinite = [t for t in self.args if t.is_infinite]
 
-        self = Add(*[t.as_leading_term(x) for t in self.args]).removeO()
+        self = self.func(*[t.as_leading_term(x) for t in self.args]).removeO()
         if not self:
             # simple leading term analysis gave us 0 but we have to send
             # back a term, so compute the leading term (via series)
             return old.compute_leading_term(x)
+        elif self is S.NaN:
+            return old.func._from_args(infinite)
         elif not self.is_Add:
             return self
         else:
-            plain = Add(*[s for s, _ in self.extract_leading_order(x)])
+            plain = self.func(*[s for s, _ in self.extract_leading_order(x)])
             rv = factor_terms(plain, fraction=False)
-            rv_fraction = factor_terms(rv, fraction=True)
+            rv_simplify = rv.simplify()
             # if it simplifies to an x-free expression, return that;
             # tests don't fail if we don't but it seems nicer to do this
-            if x not in rv_fraction.free_symbols:
-                return rv_fraction
+            if x not in rv_simplify.free_symbols:
+                if rv_simplify.is_zero and plain.is_zero is not True:
+                    return (self - plain)._eval_as_leading_term(x)
+                return rv_simplify
             return rv
 
     def _eval_adjoint(self):
-        return Add(*[t.adjoint() for t in self.args])
+        return self.func(*[t.adjoint() for t in self.args])
 
     def _eval_conjugate(self):
-        return Add(*[t.conjugate() for t in self.args])
+        return self.func(*[t.conjugate() for t in self.args])
 
     def _eval_transpose(self):
-        return Add(*[t.transpose() for t in self.args])
+        return self.func(*[t.transpose() for t in self.args])
 
     def __neg__(self):
-        return Add(*[-t for t in self.args])
+        return self.func(*[-t for t in self.args])
 
     def _sage_(self):
         s = 0
@@ -754,7 +807,7 @@ class Add(AssocOp):
             c = terms.pop(0)
         else:
             c = None
-        terms.sort(key=hash)
+        _addsort(terms)
         if c:
             terms.insert(0, c)
         return Rational(ngcd, dlcm), self._new_rawargs(*terms)
@@ -779,7 +832,8 @@ class Add(AssocOp):
 
         See docstring of Expr.as_content_primitive for more examples.
         """
-        con, prim = Add(*[_keep_coeff(*a.as_content_primitive(radical=radical)) for a in self.args]).primitive()
+        con, prim = self.func(*[_keep_coeff(*a.as_content_primitive(
+            radical=radical)) for a in self.args]).primitive()
         if radical and prim.is_Add:
             # look for common radicals that can be removed
             args = prim.args
@@ -790,8 +844,8 @@ class Add(AssocOp):
                 for ai in Mul.make_args(m):
                     if ai.is_Pow:
                         b, e = ai.as_base_exp()
-                        if e.is_Rational and b.is_Integer and b > 0:
-                            term_rads[e.q].append(int(b)**e.p)
+                        if e.is_Rational and b.is_Integer:
+                            term_rads[e.q].append(abs(int(b))**e.p)
                 if not term_rads:
                     break
                 if common_q is None:
@@ -805,7 +859,7 @@ class Add(AssocOp):
                 # process rads
                 # keep only those in common_q
                 for r in rads:
-                    for q in r.keys():
+                    for q in list(r.keys()):
                         if q not in common_q:
                             r.pop(q)
                     for q in r:
@@ -819,14 +873,14 @@ class Add(AssocOp):
                 if G:
                     G = Mul(*G)
                     args = [ai/G for ai in args]
-                    prim = G*Add(*args)
+                    prim = G*prim.func(*args)
 
         return con, prim
 
     @property
     def _sorted_args(self):
-        from sympy.utilities.misc import default_sort_key
-        return sorted(self.args, key=lambda w: default_sort_key(w))
+        from sympy.core.compatibility import default_sort_key
+        return tuple(sorted(self.args, key=lambda w: default_sort_key(w)))
 
-from mul import Mul, _keep_coeff, prod
+from .mul import Mul, _keep_coeff, prod
 from sympy.core.numbers import Rational
