@@ -1,12 +1,15 @@
+from __future__ import division
 from sympy import (Symbol, Wild, sin, cos, exp, sqrt, pi, Function, Derivative,
         abc, Integer, Eq, symbols, Add, I, Float, log, Rational, Lambda, atan2,
-        cse, cot, tan, S, Tuple, Basic, Dict, Piecewise, oo)
+        cse, cot, tan, S, Tuple, Basic, Dict, Piecewise, oo, Mul,
+        factor, nsimplify, zoo, Subs)
 from sympy.core.basic import _aresame
-from sympy.utilities.pytest import XFAIL, raises
+from sympy.utilities.pytest import XFAIL
+from sympy.abc import x, y, z
+
 
 def test_subs():
     n3 = Rational(3)
-    x = Symbol('x')
     e = x
     e = e.subs(x, n3)
     assert e == Rational(3)
@@ -16,8 +19,8 @@ def test_subs():
     e = e.subs(x, n3)
     assert e == Rational(6)
 
+
 def test_trigonometric():
-    x = Symbol('x')
     n3 = Rational(3)
     e = (sin(x)**2).diff(x)
     assert e == 2*sin(x)*cos(x)
@@ -42,17 +45,19 @@ def test_trigonometric():
     o = Symbol('o', odd=True)
     assert tan(o*x).subs(x, pi/2) == tan(o*pi/2)
 
+
 def test_powers():
-    x, y = symbols('x,y')
     assert sqrt(1 - sqrt(x)).subs(x, 4) == I
     assert (sqrt(1 - x**2)**3).subs(x, 2) == - 3*I*sqrt(3)
     assert (x**Rational(1, 3)).subs(x, 27) == 3
     assert (x**Rational(1, 3)).subs(x, -27) == 3*(-1)**Rational(1, 3)
     assert ((-x)**Rational(1, 3)).subs(x, 27) == 3*(-1)**Rational(1, 3)
     n = Symbol('n', negative=True)
-    assert (x**n).subs(x, 0) is S.Infinity
-    assert exp(-1).subs(S.Exp1, 0) is S.Infinity
+    assert (x**n).subs(x, 0) is S.ComplexInfinity
+    assert exp(-1).subs(S.Exp1, 0) is S.ComplexInfinity
     assert (x**(4.0*y)).subs(x**(2.0*y), n) == n**2.0
+    assert (2**(x + 2)).subs(2, 3) == 3**(x + 3)
+
 
 def test_logexppow():   # no eval()
     x = Symbol('x', real=True)
@@ -61,30 +66,32 @@ def test_logexppow():   # no eval()
     assert e.subs(2**x, w) != e
     assert e.subs(exp(x*log(Rational(2))), w) != e
 
+
 def test_bug():
     x1 = Symbol('x1')
     x2 = Symbol('x2')
     y = x1*x2
-    y.subs(x1, Float(3.0))
+    assert y.subs(x1, Float(3.0)) == Float(3.0)*x2
+
 
 def test_subbug1():
-    x = Symbol('x')
-    e = (x**x).subs(x, 1)
-    e = (x**x).subs(x, 1.0)
+    # see that they don't fail
+    (x**x).subs(x, 1)
+    (x**x).subs(x, 1.0)
+
 
 def test_subbug2():
     # Ensure this does not cause infinite recursion
-    x = Symbol('x')
     assert Float(7.7).epsilon_eq(abs(x).subs(x, -7.7))
 
+
 def test_dict_set():
-    x = Symbol('x')
     a, b, c = map(Wild, 'abc')
 
     f = 3*cos(4*x)
     r = f.match(a*cos(b*x))
     assert r == {a: 3, b: 4}
-    e =  a/b*sin(b*x)
+    e = a/b*sin(b*x)
     assert e.subs(r) == r[a]/r[b]*sin(r[b]*x)
     assert e.subs(r) == 3*sin(4*x) / 4
     s = set(r.items())
@@ -95,23 +102,23 @@ def test_dict_set():
     assert e.subs(r) == 3*sin(4*x) / 4
     assert x.subs(Dict((x, 1))) == 1
 
-def test_dict_ambigous():   # see #467
-    x = Symbol('x')
+
+def test_dict_ambigous():   # see issue 3566
     y = Symbol('y')
     z = Symbol('z')
 
     f = x*exp(x)
     g = z*exp(z)
 
-    df= {x:y, exp(x): y}
-    dg= {z:y, exp(z): y}
+    df = {x: y, exp(x): y}
+    dg = {z: y, exp(z): y}
 
     assert f.subs(df) == y**2
     assert g.subs(dg) == y**2
 
     # and this is how order can affect the result
-    assert f .subs(x, y) .subs(exp(x), y)  == y*exp(y)
-    assert f .subs(exp(x), y) .subs(x, y)  == y**2
+    assert f.subs(x, y).subs(exp(x), y) == y*exp(y)
+    assert f.subs(exp(x), y).subs(x, y) == y**2
 
     # length of args and count_ops are the same so
     # default_sort_key resolves ordering...if one
@@ -121,16 +128,17 @@ def test_dict_ambigous():   # see #467
     assert e.subs({x: y, y: 2}) == 5
     # here, there are no obviously clashing keys or values
     # but the results depend on the order
-    assert exp(x/2 + y).subs(dict([(exp(y + 1), 2),(x, 2)])) == exp(y + 1)
+    assert exp(x/2 + y).subs(dict([(exp(y + 1), 2), (x, 2)])) == exp(y + 1)
+
 
 def test_deriv_sub_bug3():
-    x = Symbol('x')
     y = Symbol('y')
     f = Function('f')
     pat = Derivative(f(x), x, x)
     assert pat.subs(y, y**2) == Derivative(f(x), x, x)
     assert pat.subs(y, y**2) != Derivative(f(x), x)
 
+
 def test_equality_subs1():
     f = Function('f')
     x = abc.x
@@ -138,34 +146,37 @@ def test_equality_subs1():
     res = Eq(Integer(16), x)
     assert eq.subs(f(x), 4) == res
 
+
 def test_equality_subs2():
     f = Function('f')
     x = abc.x
     eq = Eq(f(x)**2, 16)
-    assert bool(eq.subs(f(x), 3)) == False
-    assert bool(eq.subs(f(x), 4)) == True
+    assert bool(eq.subs(f(x), 3)) is False
+    assert bool(eq.subs(f(x), 4)) is True
 
-def test_issue643():
-    x = Symbol('x')
+
+def test_issue_3742():
     y = Symbol('y')
 
     e = sqrt(x)*exp(y)
-    assert e.subs(sqrt(x), 1)   == exp(y)
+    assert e.subs(sqrt(x), 1) == exp(y)
+
 
 def test_subs_dict1():
     x, y = symbols('x y')
     assert (1 + x*y).subs(x, pi) == 1 + pi*y
-    assert (1 + x*y).subs({x:pi, y:2}) == 1 + 2*pi
+    assert (1 + x*y).subs({x: pi, y: 2}) == 1 + 2*pi
 
     c2, c3, q1p, q2p, c1, s1, s2, s3 = symbols('c2 c3 q1p q2p c1 s1 s2 s3')
     test = (c2**2*q2p*c3 + c1**2*s2**2*q2p*c3 + s1**2*s2**2*q2p*c3
             - c1**2*q1p*c2*s3 - s1**2*q1p*c2*s3)
-    assert (test.subs({c1**2 : 1 - s1**2, c2**2 : 1 - s2**2, c3**3: 1 - s3**2})
+    assert (test.subs({c1**2: 1 - s1**2, c2**2: 1 - s2**2, c3**3: 1 - s3**2})
         == c3*q2p*(1 - s2**2) + c3*q2p*s2**2*(1 - s1**2)
             - c2*q1p*s3*(1 - s1**2) + c3*q2p*s1**2*s2**2 - c2*q1p*s3*s1**2)
 
+
 def test_mul():
-    x, y, z, a, b, c = symbols('x,y,z,a,b,c')
+    x, y, z, a, b, c = symbols('x y z a b c')
     A, B, C = symbols('A B C', commutative=0)
     assert (x*y*z).subs(z*x, y) == y**2
     assert (z*x).subs(1/x, z) == z*x
@@ -198,15 +209,15 @@ def test_mul():
     assert (x*A*B*A*exp(x*A*B)).subs(x*A, B) == B**2*A*exp(B*B)
     assert (x**2*A*B*A*exp(x*A*B)).subs(x*A, B) == B**3*exp(B**2)
     assert (x**3*A*exp(x*A*B)*A*exp(x*A*B)).subs(x*A, B) == \
-            x*B*exp(B**2)*B*exp(B**2)
+        x*B*exp(B**2)*B*exp(B**2)
     assert (x*A*B*C*A*B).subs(x*A*B, C) == C**2*A*B
     assert (-I*a*b).subs(a*b, 2) == -2*I
 
-    # issue 3262
+    # issue 6361
     assert (-8*I*a).subs(-2*a, 1) == 4*I
     assert (-I*a).subs(-a, 1) == I
 
-    # issue 3342
+    # issue 6441
     assert (4*x**2).subs(2*x, y) == y**2
     assert (2*4*x**2).subs(2*x, y) == 2*y**2
     assert (-x**3/9).subs(-x/3, z) == -z**2*x
@@ -216,7 +227,8 @@ def test_mul():
     assert (-2*x**3/9).subs(-2*x, z) == z*x**2/9
     assert (-2*x**3/9).subs(2*x, z) == -z*x**2/9
     assert (2*(3*x/5/7)**2).subs(3*x/5, z) == 2*(S(1)/7)**2*z**2
-    assert (4*x).subs(-2*x, z) == 4*x # try keep subs literal
+    assert (4*x).subs(-2*x, z) == 4*x  # try keep subs literal
+
 
 def test_subs_simple():
     a = symbols('a', commutative=True)
@@ -234,6 +246,7 @@ def test_subs_simple():
     assert (2*x).subs(x, 3) == 6
     assert sin(x).subs(x, 3) == sin(3)
 
+
 def test_subs_constants():
     a, b = symbols('a b', commutative=True)
     x, y = symbols('x y', commutative=False)
@@ -248,6 +261,7 @@ def test_subs_constants():
     assert (2*x*y).subs(2*x, 1) == y
     assert (2*x*y).subs(4*x, 1) == 2*x*y
 
+
 def test_subs_commutative():
     a, b, c, d, K = symbols('a b c d K', commutative=True)
 
@@ -261,6 +275,7 @@ def test_subs_commutative():
     assert (a*b*c*b*a).subs(a*b, K) == c*K**2
     assert (a**3*b**2*a).subs(a*b, K) == a**2*K**2
 
+
 def test_subs_noncommutative():
     w, x, y, z, L = symbols('w x y z L', commutative=False)
 
@@ -277,6 +292,7 @@ def test_subs_noncommutative():
     assert (w*x*y*z*x*y).subs(x*y*z, L) == w*L*x*y
     assert (w*x*y*y*w*x*x*y*x*y*y*x*y).subs(x*y, L) == w*L*y*w*x*L**2*y*L
 
+
 def test_subs_basic_funcs():
     a, b, c, d, K = symbols('a b c d K', commutative=True)
     w, x, y, z, L = symbols('w x y z L', commutative=False)
@@ -288,12 +304,14 @@ def test_subs_basic_funcs():
     assert (x**y).subs(y, L) == x**L
     assert ((a - c)/b).subs(b, K) == (a - c)/K
     assert (exp(x*y - z)).subs(x*y, L) == exp(L - z)
-    assert (a*exp(x*y - w*z) + b*exp(x*y + w*z)).subs(z, 0) == a*exp(x*y) + b*exp(x*y)
+    assert (a*exp(x*y - w*z) + b*exp(x*y + w*z)).subs(z, 0) == \
+        a*exp(x*y) + b*exp(x*y)
     assert ((a - b)/(c*d - a*b)).subs(c*d - a*b, K) == (a - b)/K
     assert (w*exp(a*b - c)*x*y/4).subs(x*y, L) == w*exp(a*b - c)*L/4
 
+
 def test_subs_wild():
-    R, S, T, U = symbols('R,S,T,U', cls=Wild)
+    R, S, T, U = symbols('R S T U', cls=Wild)
 
     assert (R*S).subs(R*S, T) == T
     assert (S*R).subs(R*S, T) == T
@@ -304,37 +322,41 @@ def test_subs_wild():
     assert (R*S**T).subs(S, U) == R*U**T
     assert (R*S**T).subs(T, U) == R*S**U
 
+
 def test_subs_mixed():
     a, b, c, d, K = symbols('a b c d K', commutative=True)
     w, x, y, z, L = symbols('w x y z L', commutative=False)
-    R, S, T, U = symbols('R,S,T,U', cls=Wild)
+    R, S, T, U = symbols('R S T U', cls=Wild)
 
     assert (a*x*y).subs(x*y, L) == a*L
     assert (a*b*x*y*x).subs(x*y, L) == a*b*L*x
     assert (R*x*y*exp(x*y)).subs(x*y, L) == R*L*exp(L)
     assert (a*x*y*y*x - x*y*z*exp(a*b)).subs(x*y, L) == a*L*y*x - L*z*exp(a*b)
     e = c*y*x*y*x**(R*S - a*b) - T*(a*R*b*S)
-    assert e.subs(x*y, L).subs(a*b, K).subs(R*S, U) == c*y*L*x**(U - K) - T*(U*K)
+    assert e.subs(x*y, L).subs(a*b, K).subs(R*S, U) == \
+        c*y*L*x**(U - K) - T*(U*K)
+
 
 def test_division():
     a, b, c = symbols('a b c', commutative=True)
     x, y, z = symbols('x y z', commutative=True)
 
-    assert (1/a).subs(a, c)  == 1/c
-    assert (1/a**2).subs(a, c)  == 1/c**2
+    assert (1/a).subs(a, c) == 1/c
+    assert (1/a**2).subs(a, c) == 1/c**2
     assert (1/a**2).subs(a, -2) == Rational(1, 4)
     assert (-(1/a**2)).subs(a, -2) == -Rational(1, 4)
 
-    assert (1/x).subs(x, z)  == 1/z
-    assert (1/x**2).subs(x, z)  == 1/z**2
+    assert (1/x).subs(x, z) == 1/z
+    assert (1/x**2).subs(x, z) == 1/z**2
     assert (1/x**2).subs(x, -2) == Rational(1, 4)
     assert (-(1/x**2)).subs(x, -2) == -Rational(1, 4)
 
-    #issue 2261
+    #issue 5360
     assert (1/x).subs(x, 0) == 1/S(0)
 
+
 def test_add():
-    a, b, c, d, x, y, t = symbols('a,b,c,d,x,y,t')
+    a, b, c, d, x, y, t = symbols('a b c d x y t')
 
     assert (a**2 - b - c).subs(a**2 - b, d) in [d - c, a**2 - b - c]
     assert (a**2 - c).subs(a**2 - c, d) == d
@@ -364,49 +386,72 @@ def test_add():
     ans = (-x*(x) - y*(-x)).expand()
     assert e.subs(-y + 1, x) == ans
 
-def test_subs_issue910():
+
+def test_subs_issue_4009():
     assert (I*Symbol('a')).subs(1, 2) == I*Symbol('a')
 
+
 def test_functions_subs():
-    x, y = map(Symbol, 'xy')
-    f, g = map(Function, 'fg')
+    x, y = symbols('x y')
+    f, g = symbols('f g', cls=Function)
     l = Lambda((x, y), sin(x) + y)
     assert (g(y, x) + cos(x)).subs(g, l) == sin(y) + x + cos(x)
     assert (f(x)**2).subs(f, sin) == sin(x)**2
     assert (f(x, y)).subs(f, log) == log(x, y)
     assert (f(x, y)).subs(f, sin) == f(x, y)
-    assert (sin(x) + atan2(x, y)).subs([[atan2, f], [sin, g]]) == f(x, y) + g(x)
+    assert (sin(x) + atan2(x, y)).subs([[atan2, f], [sin, g]]) == \
+        f(x, y) + g(x)
     assert (g(f(x + y, x))).subs([[f, l], [g, exp]]) == exp(x + sin(x + y))
 
+
 def test_derivative_subs():
-    x = Symbol('x')
     y = Symbol('y')
     f = Function('f')
     assert Derivative(f(x), x).subs(f(x), y) != 0
     assert Derivative(f(x), x).subs(f(x), y).subs(y, f(x)) == \
         Derivative(f(x), x)
-    # issues 1986, 1938
+    # issues 5085, 5037
     assert cse(Derivative(f(x), x) + f(x))[1][0].has(Derivative)
     assert cse(Derivative(f(x, y), x) +
                Derivative(f(x, y), y))[1][0].has(Derivative)
 
-def test_issue2185():
+def test_derivative_subs2():
+    x, y, z = symbols('x y z')
+    f, g = symbols('f g', cls=Function)
+    assert Derivative(f, x, y).subs(Derivative(f, x, y), g) == g
+    assert Derivative(f, y, x).subs(Derivative(f, x, y), g) == g
+    assert Derivative(f, x, y).subs(Derivative(f, x), g) == Derivative(g, y)
+    assert Derivative(f, x, y).subs(Derivative(f, y), g) == Derivative(g, x)
+    assert (Derivative(f(x, y, z), x, y, z).subs(
+                Derivative(f(x, y, z), x, z), g) == Derivative(g, y))
+    assert (Derivative(f(x, y, z), x, y, z).subs(
+                Derivative(f(x, y, z), z, y), g) == Derivative(g, x))
+    assert (Derivative(f(x, y, z), x, y, z).subs(
+                Derivative(f(x, y, z), z, y, x), g) == g)
+
+def test_derivative_subs3():
     x = Symbol('x')
+    dex = Derivative(exp(x), x)
+    assert Derivative(dex, x).subs(dex, exp(x)) == dex
+    assert dex.subs(exp(x), dex) == Derivative(exp(x), x, x)
+
+def test_issue_5284():
     A, B = symbols('A B', commutative=False)
     assert (x*A).subs(x**2*A, B) == x*A
     assert (A**2).subs(A**3, B) == A**2
     assert (A**6).subs(A**3, B) == B**2
 
+
 def test_subs_iter():
-    x, y = symbols('x,y')
     assert x.subs(reversed([[x, y]])) == y
     it = iter([[x, y]])
     assert x.subs(it) == y
     assert x.subs(Tuple((x, y))) == y
 
+
 def test_subs_dict():
     a, b, c, d, e = symbols('a b c d e')
-    x, y, z = symbols('x y z')
+    z = symbols('z')
 
     assert (2*x + y + z).subs(dict(x=1, y=2)) == 4 + z
 
@@ -422,7 +467,7 @@ def test_subs_dict():
                (cos(2*x), b),
                (exp(x), e),
                (x, d),
-               ])
+    ])
     assert expr.subs(reps) == c + a*b*sin(d*e)
 
     l = [(x, 3), (y, x**2)]
@@ -437,40 +482,39 @@ def test_subs_dict():
     l = [(y, z + 2), (z, 3)]
     assert (y - 2).subs(l) == 3
 
+
 def test_no_arith_subs_on_floats():
-    a, x, y = symbols('a,x,y')
+    a, x, y = symbols('a x y')
 
-    (x + 3).subs(x + 3, a) == a
-    (x + 3).subs(x + 2, a) == a + 1
+    assert (x + 3).subs(x + 3, a) == a
+    assert (x + 3).subs(x + 2, a) == a + 1
 
-    (x + y + 3).subs(x + 3, a) == a + y
-    (x + y + 3).subs(x + 2, a) == a + y + 1
+    assert (x + y + 3).subs(x + 3, a) == a + y
+    assert (x + y + 3).subs(x + 2, a) == a + y + 1
 
-    (x + 3.0).subs(x + 3.0, a) == a
-    (x + 3.0).subs(x + 2.0, a) == x + y + 3.0
+    assert (x + 3.0).subs(x + 3.0, a) == a
+    assert (x + 3.0).subs(x + 2.0, a) == x + 3.0
 
-    (x + y + 3.0).subs(x + 3.0, a) == a + y
-    (x + y + 3.0).subs(x + 2.0, a) == x + y + 3.0
+    assert (x + y + 3.0).subs(x + 3.0, a) == a + y
+    assert (x + y + 3.0).subs(x + 2.0, a) == x + y + 3.0
 
-@XFAIL
-def test_issue_2261() :
-    x = Symbol('x')
-    assert (1/x).subs(x, 0) == 1/S(0)
 
-def test_issue_2552():
+def test_issue_5651():
     a, b, c, K = symbols('a b c K', commutative=True)
     x, y, z = symbols('x y z')
     assert (a/(b*c)).subs(b*c, K) == a/K
     assert (a/(b**2*c**3)).subs(b*c, K) == a/(c*K**2)
     assert (1/(x*y)).subs(x*y, 2) == S.Half
     assert ((1 + x*y)/(x*y)).subs(x*y, 1) == 2
-    assert (x*y*z).subs(x*y,2) == 2*z
+    assert (x*y*z).subs(x*y, 2) == 2*z
     assert ((1 + x*y)/(x*y)/z).subs(x*y, 1) == 2/z
 
-def test_issue_2976():
+
+def test_issue_6075():
     assert Tuple(1, True).subs(1, 2) == Tuple(2, True)
 
-def test_issue_2980():
+
+def test_issue_6079():
     # since x + 2.0 == x + 2 we can't do a simple equality test
     x = symbols('x')
     assert _aresame((x + 2.0).subs(2, 3), x + 2.0)
@@ -481,36 +525,118 @@ def test_issue_2980():
     assert not _aresame(1, S(1))
     assert not _aresame(x, symbols('x', positive=True))
 
-def test_issue_1581():
+
+def test_issue_4680():
     N = Symbol('N')
     assert N.subs(dict(N=3)) == 3
 
-def test_issue_3059():
-    from sympy.abc import x, y
+
+def test_issue_6158():
     assert (x - 1).subs(1, y) == x - y
     assert (x - 1).subs(-1, y) == x + y
     assert (x - oo).subs(oo, y) == x - y
     assert (x - oo).subs(-oo, y) == x + y
 
+
 def test_Function_subs():
     from sympy.abc import x, y
-    f, g, h, i = symbols('f,g,h,i', cls=Function)
-    p = Piecewise((g(f(x,y)), x < -1), (g(x), x <= 1))
-    assert p.subs(g, h) == Piecewise((h(f(x,y)), x < -1), (h(x), x <= 1))
-    assert (f(y) + g(x)).subs({f:h,g:i}) == i(x) + h(y)
+    f, g, h, i = symbols('f g h i', cls=Function)
+    p = Piecewise((g(f(x, y)), x < -1), (g(x), x <= 1))
+    assert p.subs(g, h) == Piecewise((h(f(x, y)), x < -1), (h(x), x <= 1))
+    assert (f(y) + g(x)).subs({f: h, g: i}) == i(x) + h(y)
+
 
 def test_simultaneous_subs():
-    from sympy.abc import x, y
     reps = {x: 0, y: 0}
     assert (x/y).subs(reps) != (y/x).subs(reps)
-    assert (x/y).subs(reps, simultaneous=True) == (y/x).subs(reps, simultaneous=True)
+    assert (x/y).subs(reps, simultaneous=True) == \
+        (y/x).subs(reps, simultaneous=True)
     reps = reps.items()
     assert (x/y).subs(reps) != (y/x).subs(reps)
-    assert (x/y).subs(reps, simultaneous=True) == (y/x).subs(reps, simultaneous=True)
+    assert (x/y).subs(reps, simultaneous=True) == \
+        (y/x).subs(reps, simultaneous=True)
+    assert Derivative(x, y, z).subs(reps, simultaneous=True) == \
+        Subs(Derivative(0, y, z), (y,), (0,))
 
-def issue_3320_3322():
-    from sympy.abc import x, y
-    assert (1/(1+x/y)).subs(x/y, x) == 1/(1+x)
+
+def test_issue_6419_6421():
+    assert (1/(1 + x/y)).subs(x/y, x) == 1/(1 + x)
     assert (-2*I).subs(2*I, x) == -x
     assert (-I*x).subs(I*x, x) == -x
-    assert (-3*I*y**4).subs(3*I*y**2,x) == -x*y**2
+    assert (-3*I*y**4).subs(3*I*y**2, x) == -x*y**2
+
+
+def test_issue_6559():
+    assert (-12*x + y).subs(-x, 1) == 12 + y
+    # though this involves cse it generated a failure in Mul._eval_subs
+    x0, x1 = symbols('x0 x1')
+    e = -log(-12*sqrt(2) + 17)/24 - log(-2*sqrt(2) + 3)/12 + sqrt(2)/3
+    # XXX modify cse so x1 is eliminated and x0 = -sqrt(2)?
+    assert cse(e) == (
+        [(x0, sqrt(2))], [x0/3 - log(-12*x0 + 17)/24 - log(-2*x0 + 3)/12])
+
+
+def test_issue_5261():
+    x = symbols('x', real=True)
+    e = I*x
+    assert exp(e).subs(exp(x), y) == y**I
+    assert (2**e).subs(2**x, y) == y**I
+    eq = (-2)**e
+    assert eq.subs((-2)**x, y) == eq
+
+
+def test_issue_6923():
+    assert (-2*x*sqrt(2)).subs(2*x, y) == -sqrt(2)*y
+
+
+def test_2arg_hack():
+    N = Symbol('N', commutative=False)
+    ans = Mul(2, y + 1, evaluate=False)
+    assert (2*x*(y + 1)).subs(x, 1, hack2=True) == ans
+    assert (2*(y + 1 + N)).subs(N, 0, hack2=True) == ans
+
+
+@XFAIL
+def test_mul2():
+    """When this fails, remove things labelled "2-arg hack"
+    1) remove special handling in the fallback of subs that
+    was added in the same commit as this test
+    2) remove the special handling in Mul.flatten
+    """
+    assert (2*(x + 1)).is_Mul
+
+
+def test_noncommutative_subs():
+    x,y = symbols('x,y', commutative=False)
+    assert (x*y*x).subs([(x,x*y),(y,x)],simultaneous=True) == (x*y*x**2*y)
+
+
+def test_issue_2877():
+    f = Float(2.0)
+    assert (x + f).subs({f: 2}) == x + 2
+
+    def r(a,b,c):
+        return factor(a*x**2 + b*x + c)
+    e = r(5/6, 10, 5)
+    assert nsimplify(e) == 5*x**2/6 + 10*x + 5
+
+
+def test_issue_5910():
+    t = Symbol('t')
+    assert (1/(1 - t)).subs(t, 1) == zoo
+    n = t
+    d = t - 1
+    assert (n/d).subs(t, 1) == zoo
+    assert (-n/-d).subs(t, 1) == zoo
+
+
+def test_issue_5217():
+    s = Symbol('s')
+    z = (1 - 2*x*x)
+    w = (1 + 2*x*x)
+    q = 2*x*x*2*y*y
+    sub = {2*x*x: s}
+    assert w.subs(sub) == 1 + s
+    assert z.subs(sub) == 1 - s
+    assert q == 4*x**2*y**2
+    assert q.subs(sub) == 2*y**2*s
