@@ -14,9 +14,11 @@ from sympy.core.containers import Tuple
 from sympy.simplify import simplify, nsimplify
 from sympy.geometry.exceptions import GeometryError
 from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.complexes import im
 from .entity import GeometryEntity
 from sympy.matrices import Matrix
 from sympy.core.numbers import Float
+from sympy.core.evaluate import global_evaluate
 
 
 class Point(GeometryEntity):
@@ -37,16 +39,10 @@ class Point(GeometryEntity):
     Raises
     ======
 
-    NotImplementedError
-        When trying to create a point with more than two dimensions.
-        When `intersection` is called with object other than a Point.
     TypeError
         When trying to add or subtract points with different dimensions.
-
-    Notes
-    =====
-
-    Currently only 2-dimensional points are supported.
+        When trying to create a point with more than two dimensions.
+        When `intersection` is called with object other than a Point.
 
     See Also
     ========
@@ -74,36 +70,30 @@ class Point(GeometryEntity):
     Point(0.5, 0.25)
 
     """
-
     def __new__(cls, *args, **kwargs):
-        if iterable(args[0]):
-            args = args[0]
-        elif isinstance(args[0], Point):
+        eval = kwargs.get('evaluate', global_evaluate[0])
+        check = True
+        if isinstance(args[0], Point):
+            if not eval:
+                return args[0]
             args = args[0].args
+            check = False
+        else:
+            if iterable(args[0]):
+                args = args[0]
+            if len(args) != 2:
+                raise ValueError(
+                    "Only two dimensional points currently supported")
         coords = Tuple(*args)
-
-        if len(coords) != 2:
-            raise NotImplementedError(
-                "Only two dimensional points currently supported")
-        if kwargs.get('evaluate', True):
+        if check:
+            if any(a.is_number and im(a) for a in coords):
+                raise ValueError('Imaginary args not permitted.')
+        if eval:
             coords = coords.xreplace(dict(
                 [(f, simplify(nsimplify(f, rational=True)))
                 for f in coords.atoms(Float)]))
-
         return GeometryEntity.__new__(cls, *coords)
 
-    def __hash__(self):
-        return super(Point, self).__hash__()
-
-    def __eq__(self, other):
-        ts, to = type(self), type(other)
-        if ts is not to:
-            return False
-        return self.args == other.args
-
-    def __lt__(self, other):
-        return self.args < other.args
-
     def __contains__(self, item):
         return item == self
 
@@ -210,6 +200,9 @@ class Point(GeometryEntity):
         # Coincident points are irrelevant and can confuse this algorithm.
         # Use only unique points.
         points = list(set(points))
+        if not all(isinstance(p, Point) for p in points):
+            raise TypeError('Must pass only 3D Point objects')
+
         if len(points) == 0:
             return False
         if len(points) <= 2:
@@ -387,10 +380,7 @@ class Point(GeometryEntity):
         Point(0.5, 1.5)
 
         """
-        if prec is None:
-            coords = [x.evalf(**options) for x in self.args]
-        else:
-            coords = [x.evalf(prec, **options) for x in self.args]
+        coords = [x.evalf(prec, **options) for x in self.args]
         return Point(*coords, evaluate=False)
 
     n = evalf
