@@ -8,46 +8,67 @@ Features:
 References:
   - http://en.wikipedia.org/wiki/DPLL_algorithm
 """
+from __future__ import print_function, division
+
 from collections import defaultdict
 from heapq import heappush, heappop
 
-from sympy.core import Symbol
-from sympy import Predicate
-from sympy.logic.boolalg import conjuncts, to_cnf, to_int_repr
+from sympy import default_sort_key, ordered
+from sympy.logic.boolalg import conjuncts, to_cnf, to_int_repr, _find_predicates
 
-def dpll_satisfiable(expr):
+
+def dpll_satisfiable(expr, all_models=False):
     """
     Check satisfiability of a propositional sentence.
-    It returns a model rather than True when it succeeds
+    It returns a model rather than True when it succeeds.
+    Returns a generator of all models if all_models is True.
 
     Examples
     ========
 
-    >>> from sympy import symbols
     >>> from sympy.abc import A, B
-    >>> from sympy.logic.algorithms.dpll import dpll_satisfiable
+    >>> from sympy.logic.algorithms.dpll2 import dpll_satisfiable
     >>> dpll_satisfiable(A & ~B)
     {A: True, B: False}
     >>> dpll_satisfiable(A & ~A)
     False
 
     """
-    symbols = list(expr.atoms(Symbol, Predicate))
-    symbols_int_repr = range(1, len(symbols) + 1)
     clauses = conjuncts(to_cnf(expr))
+    if False in clauses:
+        if all_models:
+            return (f for f in [False])
+        return False
+    symbols = sorted(_find_predicates(expr), key=default_sort_key)
+    symbols_int_repr = range(1, len(symbols) + 1)
     clauses_int_repr = to_int_repr(clauses, symbols)
 
-    solver = SATSolver(clauses_int_repr, symbols_int_repr, set())
-    result = solver._find_model()
+    solver = SATSolver(clauses_int_repr, symbols_int_repr, set(), symbols)
+    models = solver._find_model()
+
+    if all_models:
+        return _all_models(models)
+
+    try:
+        return next(models)
+    except StopIteration:
+        return False
 
-    if not result:
-        return result
     # Uncomment to confirm the solution is valid (hitting set for the clauses)
     #else:
         #for cls in clauses_int_repr:
             #assert solver.var_settings.intersection(cls)
 
-    return dict((symbols[abs(lit) - 1], lit > 0) for lit in solver.var_settings)
+
+def _all_models(models):
+    satisfiable = False
+    try:
+        while True:
+            yield next(models)
+            satisfiable = True
+    except StopIteration:
+        if not satisfiable:
+            yield False
 
 
 class SATSolver(object):
@@ -57,8 +78,9 @@ class SATSolver(object):
      normal form.
     """
 
-    def __init__(self, clauses, variables, var_settings, heuristic = 'vsids', \
-                 clause_learning = 'none', INTERVAL = 500):
+    def __init__(self, clauses, variables, var_settings, symbols=None,
+                heuristic='vsids', clause_learning='none', INTERVAL=500):
+
         self.var_settings = var_settings
         self.heuristic = heuristic
         self.is_unsatisfied = False
@@ -66,6 +88,11 @@ class SATSolver(object):
         self.update_functions = []
         self.INTERVAL = INTERVAL
 
+        if symbols is None:
+            self.symbols = list(ordered(variables))
+        else:
+            self.symbols = symbols
+
         self._initialize_variables(variables)
         self._initialize_clauses(clauses)
 
@@ -132,9 +159,9 @@ class SATSolver(object):
             for lit in self.clauses[i]:
                 self.occurrence_count[lit] += 1
 
-
     def _find_model(self):
-        """Main DPLL loop.
+        """
+        Main DPLL loop. Returns a generator of models.
 
         Variables are chosen successively, and assigned to be either
         True or False. If a solution is not found with this setting,
@@ -145,10 +172,16 @@ class SATSolver(object):
         ========
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
-        >>> SATSolver([set([-1]), set([1])], set([1]), set([]))._find_model()
-        False
-        >>> SATSolver([set([1]), set([-2])], set([-2, 3]), set([]))._find_model()
-        True
+        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
+        ... set([3, -2])], set([1, 2, 3]), set([]))
+        >>> list(l._find_model())
+        [{1: True, 2: False, 3: False}, {1: True, 2: True, 3: True}]
+
+        >>> from sympy.abc import A, B, C
+        >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
+        ... set([3, -2])], set([1, 2, 3]), set([]), [A, B, C])
+        >>> list(l._find_model())
+        [{A: True, B: False, C: False}, {A: True, B: True, C: True}]
         """
 
         # We use this variable to keep track of if we should flip a
@@ -158,7 +191,7 @@ class SATSolver(object):
         # Check if unit prop says the theory is unsat right off the bat
         self._simplify()
         if self.is_unsatisfied:
-            return False
+            return
 
         # While the theory still has clauses remaining
         while True:
@@ -179,7 +212,17 @@ class SATSolver(object):
 
                 # Stopping condition for a satisfying theory
                 if 0 == lit:
-                    return True
+                    yield dict((self.symbols[abs(lit) - 1],
+                                lit > 0) for lit in self.var_settings)
+                    while self._current_level.flipped:
+                        self._undo()
+                    if len(self.levels) == 1:
+                        return
+                    flip_lit = -self._current_level.decision
+                    self._undo()
+                    self.levels.append(Level(flip_lit, flipped=True))
+                    flip_var = True
+                    continue
 
                 # Start the new decision level
                 self.levels.append(Level(lit))
@@ -201,7 +244,7 @@ class SATSolver(object):
 
                     # If we've unrolled all the way, the theory is unsat
                     if 1 == len(self.levels):
-                        return False
+                        return
 
                 # Detect and add a learned clause
                 self.add_learned_clause(self.compute_conflict())
@@ -209,10 +252,9 @@ class SATSolver(object):
                 # Try the opposite setting of the most recent decision
                 flip_lit = -self._current_level.decision
                 self._undo()
-                self.levels.append(Level(flip_lit, flipped = True))
+                self.levels.append(Level(flip_lit, flipped=True))
                 flip_var = True
 
-
     ########################
     #    Helper Methods    #
     ########################
@@ -225,8 +267,8 @@ class SATSolver(object):
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
         >>> l = SATSolver([set([1]), set([2])], set([1, 2]), set([]))
-        >>> l._find_model()
-        True
+        >>> next(l._find_model())
+        {1: True, 2: True}
         >>> l._current_level.decision
         0
         >>> l._current_level.flipped
@@ -244,8 +286,10 @@ class SATSolver(object):
 
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
         >>> l = SATSolver([set([1]), set([-1])], set([1]), set([]))
-        >>> l._find_model()
-        False
+        >>> try:
+        ...     next(l._find_model())
+        ... except StopIteration:
+        ...     pass
         >>> l._clause_sat(0)
         False
         >>> l._clause_sat(1)
@@ -265,9 +309,8 @@ class SATSolver(object):
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
         >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
         ... set([3, -2])], set([1, 2, 3]), set([]))
-        >>> l._find_model()
-        True
-
+        >>> next(l._find_model())
+        {1: True, 2: False, 3: False}
         >>> l._is_sentinel(2, 3)
         True
         >>> l._is_sentinel(-3, 1)
@@ -290,17 +333,18 @@ class SATSolver(object):
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
         >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
         ... set([3, -2])], set([1, 2, 3]), set([]))
-        >>> l._find_model()
-        True
-
+        >>> next(l._find_model())
+        {1: True, 2: False, 3: False}
         >>> l.var_settings
         set([-3, -2, 1])
 
         >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
         ... set([3, -2])], set([1, 2, 3]), set([]))
         >>> l._assign_literal(-1)
-        >>> l._find_model()
-        False
+        >>> try:
+        ...     next(l._find_model())
+        ... except StopIteration:
+        ...     pass
         >>> l.var_settings
         set([-1])
         """
@@ -338,15 +382,12 @@ class SATSolver(object):
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
         >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
         ... set([3, -2])], set([1, 2, 3]), set([]))
-        >>> l._find_model()
-        True
-
+        >>> next(l._find_model())
+        {1: True, 2: False, 3: False}
         >>> level = l._current_level
         >>> level.decision, level.var_settings, level.flipped
         (-3, set([-3, -2]), False)
-
         >>> l._undo()
-
         >>> level = l._current_level
         >>> level.decision, level.var_settings, level.flipped
         (0, set([1]), False)
@@ -360,7 +401,6 @@ class SATSolver(object):
         # Pop the level off the stack
         self.levels.pop()
 
-
     #########################
     #      Propagation      #
     #########################
@@ -422,19 +462,10 @@ class SATSolver(object):
         """Initialize the data structures needed for the VSIDS heuristic."""
         self.lit_heap = []
         self.lit_scores = {}
-        def _nfloat(a):
-            """Return negative, float value of a.
 
-            If a is zero, don't negate it as this leads to 0.0
-            in Python 2.5. The calls to this can be dropped when
-            support for 2.5 is dropped."""
-            if a:
-                return -float(a)
-            else:
-                return 0.0
         for var in range(1, len(self.variable_set)):
-            self.lit_scores[var] = _nfloat(self.occurrence_count[var])
-            self.lit_scores[-var] = _nfloat(self.occurrence_count[-var])
+            self.lit_scores[var] = float(-self.occurrence_count[var])
+            self.lit_scores[-var] = float(-self.occurrence_count[-var])
             heappush(self.lit_heap, (self.lit_scores[var], var))
             heappush(self.lit_heap, (self.lit_scores[-var], -var))
 
@@ -544,11 +575,9 @@ class SATSolver(object):
         for lit in cls:
             self.lit_scores[lit] += 1
 
-
     ########################
     #   Clause Learning    #
     ########################
-
     def _simple_add_learned_clause(self, cls):
         """Add a new clause to the theory.
 
@@ -594,8 +623,8 @@ class SATSolver(object):
         >>> from sympy.logic.algorithms.dpll2 import SATSolver
         >>> l = SATSolver([set([2, -3]), set([1]), set([3, -3]), set([2, -2]),
         ... set([3, -2])], set([1, 2, 3]), set([]))
-        >>> l._find_model()
-        True
+        >>> next(l._find_model())
+        {1: True, 2: False, 3: False}
         >>> l._simple_compute_conflict()
         [3]
         """
@@ -605,13 +634,14 @@ class SATSolver(object):
         """Clean up learned clauses."""
         pass
 
+
 class Level(object):
     """
     Represents a single level in the DPLL algorithm, and contains
     enough information for a sound backtracking procedure.
     """
 
-    def __init__(self, decision, flipped = False):
+    def __init__(self, decision, flipped=False):
         self.decision = decision
         self.var_settings = set()
         self.flipped = flipped
