@@ -71,7 +71,7 @@ def diff(ctx, f, x, n=1, **options):
     an integer `n \ge 0`, the `n`-th derivative `f^{(n)}(x)`.
     A few basic examples are::
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>> diff(lambda x: x**2 + x, 1.0)
         3.0
@@ -243,7 +243,7 @@ def diffs(ctx, f, x, n=None, **options):
 
     **Examples**
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15
         >>> nprint(list(diffs(cos, 1, 5)))
         [0.540302, -0.841471, -0.540302, 0.841471, 0.540302, -0.841471]
@@ -264,7 +264,7 @@ def diffs(ctx, f, x, n=None, **options):
         n = int(n)
     if options.get('method', 'step') != 'step':
         k = 0
-        while k < n:
+        while k < n + 1:
             yield ctx.diff(f, x, k, **options)
             k += 1
         return
@@ -320,7 +320,7 @@ def diffs_prod(ctx, factors):
 
     **Examples**
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>> f = lambda x: exp(x)*cos(x)*sin(x)
         >>> u = diffs(f, 1)
@@ -408,7 +408,7 @@ def diffs_exp(ctx, fdiffs):
     The derivatives of the gamma function can be computed using
     logarithmic differentiation::
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>>
         >>> def diffs_loggamma(x):
@@ -476,7 +476,7 @@ def differint(ctx, f, x, n=1, x0=0):
     monomial `x^p`, which may be used as a reference. For example,
     the following gives a half-derivative (order 0.5)::
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>> x = mpf(3); p = 2; n = 0.5
         >>> differint(lambda t: t**p, x, n)
@@ -524,7 +524,7 @@ def diffun(ctx, f, n=1, **options):
     Given a function `f`, returns a function `g(x)` that evaluates the nth
     derivative `f^{(n)}(x)`::
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>> cos2 = diffun(sin)
         >>> sin2 = diffun(sin, 4)
@@ -549,7 +549,7 @@ def taylor(ctx, f, x, n, **options):
     Produces a degree-`n` Taylor polynomial around the point `x` of the
     given function `f`. The coefficients are returned as a list.
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>> nprint(chop(taylor(sin, 0, 5)))
         [0.0, 1.0, 0.0, -0.166667, 0.0, 0.00833333]
@@ -600,7 +600,7 @@ def pade(ctx, a, L, M):
     from G.A. Baker 'Essentials of Pade Approximants' Academic Press,
     Ch.1A)::
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> mp.dps = 15; mp.pretty = True
         >>> one = mpf(1)
         >>> def f(x):
@@ -617,7 +617,8 @@ def pade(ctx, a, L, M):
     """
     # To determine L+1 coefficients of P and M coefficients of Q
     # L+M+1 coefficients of A must be provided
-    assert(len(a) >= L+M+1)
+    if len(a) < L+M+1:
+        raise ValueError("L+M+1 Coefficients should be provided")
 
     if M == 0:
         if L == 0:
