@@ -495,7 +495,7 @@ def bernfrac(n):
 
     The first few Bernoulli numbers are exactly::
 
-        >>> from mpmath import *
+        >>> from sympy.mpmath import *
         >>> for n in range(15):
         ...     p, q = bernfrac(n)
         ...     print("%s %s/%s" % (n, p, q))
@@ -552,7 +552,7 @@ def bernfrac(n):
        http://mathworld.wolfram.com/vonStaudt-ClausenTheorem.html
 
     2. The Bernoulli Number Page:
-       http://www.bernoulli.org/
+       http://bernoulli.org/
 
     """
     n = int(n)
@@ -1213,7 +1213,7 @@ def mpf_zeta(s, prec, rnd=round_fast, alt=0):
         a = mpf_gamma(y, wp)
         b = mpf_zeta(y, wp)
         c = mpf_sin_pi(mpf_shift(s, -1), wp)
-        wp2 = wp + (exp+bc)
+        wp2 = wp + max(0,exp+bc)
         pi = mpf_pi(wp+wp2)
         d = mpf_div(mpf_pow(mpf_shift(pi, 1), s, wp2), pi, wp2)
         return mpf_mul(a,mpf_mul(b,mpf_mul(c,d,wp),wp),prec,rnd)
@@ -1309,7 +1309,7 @@ def mpc_zeta(s, prec, rnd=round_fast, alt=0, force=False):
         rsign, rman, rexp, rbc = re
         isign, iman, iexp, ibc = im
         mag = max(rexp+rbc, iexp+ibc)
-        wp2 = wp + mag
+        wp2 = wp + max(0, mag)
         pi = mpf_pi(wp+wp2)
         pi2 = (mpf_shift(pi, 1), fzero)
         d = mpc_div_mpf(mpc_pow(pi2, s, wp2), pi, wp2)
@@ -1406,7 +1406,8 @@ def primesieve(n):
     return sieve, primes, mult
 
 def zetasum_sieved(critical_line, sre, sim, a, n, wp):
-    assert a >= 1
+    if a < 1:
+        raise ValueError("a cannot be less than 1")
     sieve, primes, mult = primesieve(a+n)
     basic_powers = {}
     one = MPZ_ONE << wp
@@ -2378,4 +2379,3 @@ def mpf_gamma_int(n, prec, rnd=round_fast):
     if n < SMALL_FACTORIAL_CACHE_SIZE:
         return mpf_pos(small_factorial_cache[n-1], prec, rnd)
     return mpf_gamma(from_int(n), prec, rnd)
-
