@@ -6,14 +6,16 @@ Todo:
 * Update tests.
 """
 
+from __future__ import print_function, division
+
 import math
 
 from sympy import Integer, log, Mul, Add, Pow, conjugate
 from sympy.core.basic import sympify
-from sympy.matrices.matrices import Matrix, zeros
+from sympy.core.compatibility import string_types, xrange
+from sympy.matrices import Matrix, zeros
 from sympy.printing.pretty.stringpict import prettyForm
 
-
 from sympy.physics.quantum.hilbert import ComplexSpace
 from sympy.physics.quantum.state import Ket, Bra, State
 
@@ -42,6 +44,7 @@ __all__ = [
 # Qubit Classes
 #-----------------------------------------------------------------------------
 
+
 class QubitState(State):
     """Base class for Qubit and QubitBra."""
 
@@ -57,7 +60,7 @@ class QubitState(State):
             return args[0].qubit_values
 
         # Turn strings into tuple of strings
-        if len(args) == 1 and isinstance(args[0], basestring):
+        if len(args) == 1 and isinstance(args[0], string_types):
             args = tuple(args[0])
 
         args = sympify(args)
@@ -65,7 +68,8 @@ class QubitState(State):
         # Validate input (must have 0 or 1 input)
         for element in args:
             if not (element == 1 or element == 0):
-                raise ValueError("Qubit values must be 0 or 1, got: %r" % element)
+                raise ValueError(
+                    "Qubit values must be 0 or 1, got: %r" % element)
         return args
 
     @classmethod
@@ -98,7 +102,7 @@ class QubitState(State):
         return self.dimension
 
     def __getitem__(self, bit):
-        return self.qubit_values[int(self.dimension-bit-1)]
+        return self.qubit_values[int(self.dimension - bit - 1)]
 
     #-------------------------------------------------------------------------
     # Utility methods
@@ -108,7 +112,7 @@ class QubitState(State):
         """Flip the bit(s) given."""
         newargs = list(self.qubit_values)
         for i in bits:
-            bit = int(self.dimension-i-1)
+            bit = int(self.dimension - i - 1)
             if newargs[bit] == 1:
                 newargs[bit] = 0
             else:
@@ -171,7 +175,6 @@ class Qubit(QubitState, Ket):
         1
     """
 
-
     @classmethod
     def dual_class(self):
         return QubitBra
@@ -212,12 +215,12 @@ class Qubit(QubitState, Ket):
         #qubit
         sorted_idx = list(indices)
         if len(sorted_idx) == 0:
-            sorted_idx = range(0, self.nqubits)
+            sorted_idx = list(range(0, self.nqubits))
         sorted_idx.sort()
 
         #trace out for each of index
         new_mat = self*bra
-        for i in xrange(len(sorted_idx)-1, -1, -1):
+        for i in xrange(len(sorted_idx) - 1, -1, -1):
             # start from tracing out from leftmost qubit
             new_mat = self._reduced_density(new_mat, int(sorted_idx[i]))
 
@@ -244,13 +247,14 @@ class Qubit(QubitState, Ket):
 
         for i in xrange(new_size):
             for j in xrange(new_size):
-                for k in xrange(2):
+                for k in range(2):
                     col = find_index_that_is_projected(j, k, qubit)
                     row = find_index_that_is_projected(i, k, qubit)
-                    new_matrix[i,j] += old_matrix[row,col]
+                    new_matrix[i, j] += old_matrix[row, col]
 
         return new_matrix
 
+
 class QubitBra(QubitState, Bra):
     """A multi-qubit bra in the computational (z) basis.
 
@@ -286,16 +290,17 @@ class IntQubitState(QubitState):
         # that integer with the minimal number of bits.
         if len(args) == 1 and args[0] > 1:
             #rvalues is the minimum number of bits needed to express the number
-            rvalues = reversed(xrange(bitcount(abs(args[0]))))
-            qubit_values = [(args[0]>>i)&1 for i in rvalues]
+            rvalues = reversed(range(bitcount(abs(args[0]))))
+            qubit_values = [(args[0] >> i) & 1 for i in rvalues]
             return QubitState._eval_args(qubit_values)
         # For two numbers, the second number is the number of bits
         # on which it is expressed, so IntQubit(0,5) == |00000>.
         elif len(args) == 2 and args[1] > 1:
             need = bitcount(abs(args[0]))
             if args[1] < need:
-                raise ValueError('cannot represent %s with %s bits' % (args[0], args[1]))
-            qubit_values = [(args[0]>>i)&1 for i in reversed(range(args[1]))]
+                raise ValueError(
+                    'cannot represent %s with %s bits' % (args[0], args[1]))
+            qubit_values = [(args[0] >> i) & 1 for i in reversed(range(args[1]))]
             return QubitState._eval_args(qubit_values)
         else:
             return QubitState._eval_args(args)
@@ -306,7 +311,7 @@ class IntQubitState(QubitState):
         n = 1
         for i in reversed(self.qubit_values):
             number += n*i
-            n = n<<1
+            n = n << 1
         return number
 
     def _print_label(self, printer, *args):
@@ -372,6 +377,8 @@ class IntQubit(IntQubitState, Qubit):
     def dual_class(self):
         return IntQubitBra
 
+    def _eval_innerproduct_IntQubitBra(self, bra, **hints):
+        return Qubit._eval_innerproduct_QubitBra(self, bra)
 
 class IntQubitBra(IntQubitState, QubitBra):
     """A qubit bra that store integers as binary numbers in qubit values."""
@@ -438,24 +445,25 @@ def matrix_to_qubit(matrix):
     result = 0
     for i in range(mlistlen):
         if ket:
-            element = matrix[i,0]
+            element = matrix[i, 0]
         else:
-            element = matrix[0,i]
+            element = matrix[0, i]
         if format == 'numpy' or format == 'scipy.sparse':
             element = complex(element)
         if element != 0.0:
             # Form Qubit array; 0 in bit-locations where i is 0, 1 in
             # bit-locations where i is 1
-            qubit_array = [int(i & (1<<x) != 0) for x in range(nqubits)]
+            qubit_array = [int(i & (1 << x) != 0) for x in range(nqubits)]
             qubit_array.reverse()
             result = result + element*cls(*qubit_array)
 
     # If sympy simplified by pulling out a constant coefficient, undo that.
-    if isinstance(result, (Mul,Add,Pow)):
+    if isinstance(result, (Mul, Add, Pow)):
         result = result.expand()
 
     return result
 
+
 def matrix_to_density(mat):
     """
     Works by finding the eigenvectors and eigenvalues of the matrix.
@@ -464,12 +472,14 @@ def matrix_to_density(mat):
     """
     from sympy.physics.quantum.density import Density
     eigen = mat.eigenvects()
-    args = [[matrix_to_qubit(Matrix([vector,])), x[0]] for x in eigen for vector in x[2] if x[0] != 0]
+    args = [[matrix_to_qubit(Matrix(
+        [vector, ])), x[0]] for x in eigen for vector in x[2] if x[0] != 0]
     if (len(args) == 0):
         return 0
     else:
         return Density(*args)
 
+
 def qubit_to_matrix(qubit, format='sympy'):
     """Coverts an Add/Mul of Qubit objects into it's matrix representation
 
@@ -526,7 +536,7 @@ def measure_all(qubit, format='sympy', normalize=True):
         if normalize:
             m = m.normalized()
 
-        size = max(m.shape) # Max of shape to account for bra or ket
+        size = max(m.shape)  # Max of shape to account for bra or ket
         nqubits = int(math.log(size)/math.log(2))
         for i in range(size):
             if m[i] != 0.0:
@@ -683,8 +693,8 @@ def _get_possible_outcomes(m, bits):
 
     # This is filled with loads of dirty binary tricks...You have been warned
 
-    size = max(m.shape) # Max of shape to account for bra or ket
-    nqubits = int(math.log(size,2)+.1) # Number of qubits possible
+    size = max(m.shape)  # Max of shape to account for bra or ket
+    nqubits = int(math.log(size, 2) + .1)  # Number of qubits possible
 
     # Make the output states and put in output_matrices, nothing in them now.
     # Each state will represent a possible outcome of the measurement
@@ -692,7 +702,7 @@ def _get_possible_outcomes(m, bits):
     # bits return 0. and output_matrices[1] is the matrix for only the 0th
     # bit being true
     output_matrices = []
-    for i in range(1<<len(bits)):
+    for i in range(1 << len(bits)):
         output_matrices.append(zeros(2**nqubits, 1))
 
     # Bitmasks will help sort how to determine possible outcomes.
@@ -700,15 +710,15 @@ def _get_possible_outcomes(m, bits):
     # it will determine which state that index belongs to
     bit_masks = []
     for bit in bits:
-        bit_masks.append(1<<bit)
+        bit_masks.append(1 << bit)
 
     # Make possible outcome states
     for i in range(2**nqubits):
-        trueness = 0 # This tells us to which output_matrix this value belongs
+        trueness = 0  # This tells us to which output_matrix this value belongs
         # Find trueness
         for j in range(len(bit_masks)):
-            if i&bit_masks[j]:
-                trueness += j+1
+            if i & bit_masks[j]:
+                trueness += j + 1
         # Put the value in the correct output matrix
         output_matrices[trueness][i] = m[i]
     return output_matrices
@@ -751,7 +761,7 @@ def measure_all_oneshot(qubit, format='sympy'):
             if total > random_number:
                 break
             result += 1
-        return Qubit(IntQubit(result, int(math.log(max(m.shape),2)+.1)))
+        return Qubit(IntQubit(result, int(math.log(max(m.shape), 2) + .1)))
     else:
         raise NotImplementedError(
             "This function can't handle non-sympy matrix formats yet"
