@@ -80,15 +80,15 @@ and to see the results of some benchmarks.
 
 """
 
+from __future__ import print_function, division
+
 import ctypes
 from sympy import Symbol, cse, sympify
 from sympy.utilities.lambdify import lambdastr as getlambdastr
 from sympy.external import import_module
 
-numpy = import_module('numpy')
-
 libtccpath = './libtcc.so'
-dps = 17 # decimal places of float precision
+dps = 17  # decimal places of float precision
 # load libtcc TODO: better Windows support
 try:
     libtcc = ctypes.cdll.LoadLibrary(libtccpath)
@@ -97,6 +97,36 @@ except OSError:
 if not libtcc:
     raise ImportError('Could not load libtcc')
 
+
+def __getClosePos(expr, braces, stopchar):
+    """
+    Returns the closing position of the expression which is either the first occurrence of a character
+    in stopchar that is not in braces, the first unmatched closing brace or the end of the expression.
+
+    Examples
+    ========
+
+    >>> from sympy.utilities.compilef import __getClosePos
+    >>> __getClosePos('3*x', '()', '+-')
+    2
+    >>> __getClosePos('3 + x) + 2', '()', '+-')
+    2
+    >>> __getClosePos('(3 + x)*y) + 4', '()', '+-')
+    9
+    """
+    openbraces = 0
+    for i, char in enumerate(expr):
+        if char == braces[0]:
+            openbraces += 1
+        elif char == braces[1]:
+            if not openbraces:  # happens when operator is in braces
+                return i
+            openbraces -= 1
+        elif char in stopchar and not openbraces:
+            return i
+    return i
+
+
 def __getLeftRight(expr, index, oplength=1, stopchar='+-'):
     """
     Gets the expressions to the left and right of an operator.
@@ -107,55 +137,14 @@ def __getLeftRight(expr, index, oplength=1, stopchar='+-'):
 
     """
     # assumes correct syntax
-    # TODO: never repeat yourself
     # get left expression
-    left = ''
-    openbraces = 0
-    for char in reversed(expr[:index]):
-        if char == ' ': # skip whitespaces but keep them
-            left = char + left
-            continue
-        elif char == ')':
-            openbraces += 1
-            left = char + left
-        elif char == '(':
-            if not openbraces: # happens when operator is in braces
-                break
-            openbraces -= 1
-            left = char + left
-        elif char in stopchar:
-            if openbraces:
-                left = char + left
-                continue
-            else:
-                break
-        else:
-            left = char + left
-    # get right expression
-    right = ''
-    openbraces = 0
-    for char in expr[index+oplength:]:
-        if char == ' ': # skip whitespaces but keep them
-            right += char
-            continue
-        elif char == '(':
-            openbraces += 1
-            right += char
-        elif char == ')':
-            if not openbraces: # happens when operator is in braces
-                break
-            openbraces -= 1
-            right += char
-        elif char in stopchar:
-            if openbraces:
-                right += char
-                continue
-            else:
-                break
-        else:
-            right += char
+    i = __getClosePos(expr[:index][::-1], ")(", stopchar)
+    left = expr[index-i:index]
+    i = __getClosePos(expr[index + oplength:], "()", stopchar)
+    right = expr[index + oplength:index + oplength + i]
     return (left, right)
 
+
 def cexpr(pyexpr):
     """
     Python math expression string -> C expression string
@@ -174,6 +163,7 @@ def cexpr(pyexpr):
     # TODO: avoid integer division
     return pyexpr
 
+
 def _gentmpvars():
     """
     Generate symbols tmp1, tmp2, ... infinitely.
@@ -183,6 +173,7 @@ def _gentmpvars():
         i += 1
         yield Symbol('tmp' + str(i))
 
+
 def genfcode(lambdastr, use_cse=False):
     """
     Python lambda string -> C function code
@@ -206,12 +197,14 @@ def genfcode(lambdastr, use_cse=False):
     else:
         # eliminate common subexpressions
         subs, finalexpr = cse(sympify(fstr), _gentmpvars())
-        assert len(finalexpr) == 1
+        if len(finalexpr) != 1:
+            raise ValueError("Length should be 1")
         vardec = ''
         cfstr = ''
         for symbol, expr in subs:
             vardec += '    double %s;\n' % symbol.name
-            cfstr += '    %s = %s;\n' % (symbol.name, cexpr(str(expr.evalf(dps))))
+            cfstr += '    %s = %s;\n' % (
+                symbol.name, cexpr(str(expr.evalf(dps))))
         cfstr = vardec + cfstr
         finalexpr = cexpr(str(finalexpr[0].evalf(dps)))
     # generate C code
@@ -224,6 +217,7 @@ inline double f(%s)
 """ % (cvarstr, cfstr, finalexpr)
     return code
 
+
 def __run(cmd):
     """
     Checks the exit code of a ran command.
@@ -231,6 +225,7 @@ def __run(cmd):
     if not cmd == 0:
         raise RuntimeError('could not run libtcc command')
 
+
 def _compile(code, argcount=None, fname='f', fprototype=None):
     """
     C code with function -> compiled function
@@ -243,16 +238,17 @@ def _compile(code, argcount=None, fname='f', fprototype=None):
     if fprototype:
         fprototype = ctypes.CFUNCTYPE(*fprototype)
     else:
-        assert argcount, 'need argcount if no prototype is specified'
-        fprototype = ctypes.CFUNCTYPE(*[ctypes.c_double]*(argcount+1))
+        if not argcount:
+            raise ValueError("need argcount if no prototype is specified")
+        fprototype = ctypes.CFUNCTYPE(*[ctypes.c_double]*(argcount + 1))
     # see libtcc.h for API documentation
     tccstate = libtcc.tcc_new()
-    __run(libtcc.tcc_set_output_type(tccstate, 0)) # output to memory
+    __run(libtcc.tcc_set_output_type(tccstate, 0))  # output to memory
     ##print libtcc.tcc_add_library_path(tccstate, mathh) # could be dropped
-    __run(libtcc.tcc_add_library(tccstate, 'm')) # use math.h FIXME: Windows
+    __run(libtcc.tcc_add_library(tccstate, 'm'))  # use math.h FIXME: Windows
     # compile string
     __run(libtcc.tcc_compile_string(tccstate, code))
-    __run(libtcc.tcc_relocate(tccstate)) # fails if link error
+    __run(libtcc.tcc_relocate(tccstate))  # fails if link error
     # create C variable to get result
     symbol = ctypes.c_long()
     __run(libtcc.tcc_get_symbol(tccstate, ctypes.byref(symbol), fname))
@@ -260,13 +256,15 @@ def _compile(code, argcount=None, fname='f', fprototype=None):
     return fprototype(symbol.value)
 
 # expr needs to work with lambdastr
+
+
 def clambdify(args, expr, **kwargs):
     """
     SymPy expression -> compiled function
 
     Supports all standard C math functions, pi and e.
 
-    >>> from sympy import symbols, sqrt
+    >>> from sympy import sqrt
     >>> from sympy.abc import x, y
     >>> cf = clambdify((x,y), sqrt(x*y))
     >>> cf(0.5, 4)
@@ -287,6 +285,7 @@ def clambdify(args, expr, **kwargs):
     # compile code
     return _compile(code, len(args))
 
+
 def frange(*args, **kwargs):
     """
     frange(lambdastr, [start,] stop[, step]) -> ctypes double array
@@ -302,7 +301,7 @@ def frange(*args, **kwargs):
     >>> frange('lambda x: sqrt(x)', 1, 4) # doctest: +ELLIPSIS
     <__main__.c_double_Array_3 object at ...>
     >>> for i in _:
-    ...     print i
+    ...     print(i)
     ...
     1.0
     1.41421356237
@@ -324,24 +323,25 @@ def frange(*args, **kwargs):
         stop = args[2]
     if len(args) == 4:
         step = args[3]
-    assert start + step != start, \
-           'step is too small and would cause an infinite loop'
+    if start + step == start:
+        raise ValueError("step is too small and would cause an infinite loop")
     # determine length of resulting array
     # TODO: do this better
     length = stop - start
     if length % step == 0:
-        length = length/step - 1 # exclude last one
+        length = length/step - 1  # exclude last one
     else:
         length = length//step
     if step > 0:
         if start < stop:
-            length += 1 # include first one
+            length += 1  # include first one
     else:
         if start > stop:
-            length += 1 # include first one
+            length += 1  # include first one
     if length < 0:
         length = 0
-    assert length == int(length)
+    if length != int(length):
+        raise ValueError("length should be an integer")
     length = int(length)
     # create array
     a = (ctypes.c_double * length)()
@@ -373,6 +373,7 @@ void evalonrange(double *result, int n)
     # return ctypes array with results
     return a
 
+
 def evalonarray(lambdastr, array, length=None, **kwargs):
     """
     Evaluates a function on an array using machine code.
@@ -383,15 +384,15 @@ def evalonarray(lambdastr, array, length=None, **kwargs):
     array will be overwritten! Make a copy before to avoid this.
     """
     # interpret arguments
-    if hasattr(array,  'ctypes'): # numpy array
+    if hasattr(array, 'ctypes'):  # numpy array
         pointer = array.ctypes.get_as_parameter()
         length = len(array)
-    elif isinstance(array,  ctypes.Array): # ctypes array
+    elif isinstance(array, ctypes.Array):  # ctypes array
         pointer = ctypes.byref(array)
         length = len(array)
-    elif isinstance(array,  ctypes.c_void_p): # ctypes pointer FIXME
+    elif isinstance(array, ctypes.c_void_p):  # ctypes pointer FIXME
         pointer = array
-        assert isinstance(length,  int) and not length < 0
+        assert isinstance(length, int) and not length < 0
     else:
         raise ValueError('array type not recognized')
     # generate code
@@ -414,7 +415,7 @@ void evalonarray(double *array, int length)
 
 """ % genfcode(lambdastr, **kwargs)
     # compile an run on array
-    run = _compile(code,  fname='evalonarray',
+    run = _compile(code, fname='evalonarray',
                    fprototype=[None, ctypes.c_void_p, ctypes.c_int])
     run(pointer, length)
 
@@ -425,10 +426,12 @@ void evalonarray(double *array, int length)
 from sympy import sqrt, pi, lambdify
 from math import exp as _exp, cos as _cos, sin as _sin
 
+
 def test_cexpr():
     expr = '1/(g(x)*3.5)**(x - a**x)/(x**2 + a)'
     assert cexpr(expr).replace(' ', '') == \
-           '1/pow((g(x)*3.5),(x-pow(a,x)))/(pow(x,2)+a)'
+        '1/pow((g(x)*3.5),(x-pow(a,x)))/(pow(x,2)+a)'
+
 
 def test_clambdify():
     x = Symbol('x')
@@ -438,42 +441,58 @@ def test_clambdify():
     pf1 = lambdify((x, y), f1, 'math')
     cf1 = clambdify((x, y), f1)
     for i in xrange(10):
-        assert cf1(i, 10 - i) == pf1(i, 10 - i)
+        if cf1(i, 10 - i) != pf1(i, 10 - i):
+            raise ValueError("Values should be equal")
     f2 = (x - y) / z * pi
     pf2 = lambdify((x, y, z), f2, 'math')
     cf2 = clambdify((x, y, z), f2)
-    assert round(pf2(1, 2, 3),  14) == round(cf2(1, 2, 3),  14)
+    if round(pf2(1, 2, 3), 14) != round(cf2(1, 2, 3), 14):
+        raise ValueError("Values should be equal")
     # FIXME: slight difference in precision
 
+
 def test_frange():
     fstr = 'lambda x: _exp(x)*_cos(x)**x'
     f = eval(fstr)
     a = frange(fstr, 30, 168, 3)
     args = range(30, 168, 3)
-    assert len(a) == len(args)
+    if len(a) != len(args):
+        raise ValueError("Lengths should be equal")
     for i in xrange(len(a)):
-        assert a[i] == f(args[i])
-    assert len(frange('lambda x: x', 0, -10000)) == 0
-    assert len(frange('lambda x: x', -1, -1, 0.0001)) == 0
+        if a[i] != f(args[i]):
+            raise ValueError("Values should be equal")
+    if len(frange('lambda x: x', 0, -10000)) != 0:
+        raise ValueError("Length should be 0")
+    if len(frange('lambda x: x', -1, -1, 0.0001)) != 0:
+        raise ValueError("Length should be 0")
     a = frange('lambda x: x', -5, 5, 0.1)
     b = range(-50, 50)
-    assert len(a) == len(b)
+    if len(a) != len(b):
+        raise ValueError("Lengths should be equal")
     for i in xrange(len(a)):
-        assert int(round(a[i]*10)) == b[i]
+        if int(round(a[i]*10)) != b[i]:
+            raise ValueError("Values should be equal")
     a = frange('lambda x: x', 17, -9, -3)
     b = range(17, -9, -3)
-    assert len(a) == len(b)
+    if len(a) != len(b):
+        raise ValueError("Lengths should be equal")
     for i in xrange(len(a)):
-        assert a[i] == b[i]
+        if a[i] != b[i]:
+            raise ValueError("a and b should be equal")
     a = frange('lambda x: x', 2.7, -3.1, -1.01)
     b = range(270, -310, -101)
-    assert len(a) == len(b)
+    if len(a) != len(b):
+        raise ValueError("Lengths should be equal")
     for i in xrange(len(a)):
-        assert int(round(a[i]*100)) == b[i]
+        if int(round(a[i]*100)) != b[i]:
+            raise ValueError("Values should be equal")
     assert frange('lambda x: x', 0.2, 0.1, -0.1)[0] == 0.2
-    assert len(frange('lambda x: x', 0)) == 0
-    assert len(frange('lambda x: x', 1000, -1)) == 0
-    assert len(frange('lambda x: x', -1.23, 3.21, -0.0000001)) == 0
+    if len(frange('lambda x: x', 0)) != 0:
+        raise ValueError("Length should be 0")
+    if len(frange('lambda x: x', 1000, -1)) != 0:
+        raise ValueError("Length should be 0")
+    if len(frange('lambda x: x', -1.23, 3.21, -0.0000001)) != 0:
+        raise ValueError("Length should be 0")
     try:
         frange()
         assert False
@@ -485,21 +504,27 @@ def test_frange():
     except TypeError:
         pass
 
+
 def test_evalonarray_ctypes():
     a = frange('lambda x: x', 10)
     evalonarray('lambda x: _sin(x)', a)
     for i, j in enumerate(a):
-        assert _sin(i) == j
+        if _sin(i) != j:
+            raise ValueError("Values should be equal")
 # TODO: test for ctypes pointers
 ##    evalonarray('lambda x: asin(x)', ctypes.byref(a), len(a))
 ##    for i, j in enumerater(a):
 ##        print j
 
+
 def test_evalonarray_numpy():
+    numpy = import_module('numpy')
     a = numpy.arange(10, dtype=float)
-    evalonarray('lambda x: x + 1',  a)
+    evalonarray('lambda x: x + 1', a)
     for i, j in enumerate(a):
-        assert float(i + 1) == j
+        if float(i + 1) != j:
+            raise ValueError("Values should be equal")
+
 
 def test_use_cse():
     args = ('lambda x: sqrt(x + 1)**sqrt(x + 1)', 1, 10)
@@ -507,9 +532,12 @@ def test_use_cse():
     kwargs = {}
     kwargs['use_cse'] = True
     b = frange(*args, **kwargs)
-    assert len(a) == len(b)
+    if len(a) != len(b):
+        raise ValueError("Lengths should be equal")
     for i in xrange(len(a)):
-        assert a[i] == b[i]
+        if a[i] != b[i]:
+            raise ValueError("a and b should be equal")
+
 
 def benchmark():
     """
@@ -527,7 +555,8 @@ def benchmark():
         global cf, pf, psyf
         start = time()
         cf = clambdify(var, f)
-        print 'compile time (including sympy overhead): %f s' % (time() - start)
+        print('compile time (including sympy overhead): %f s' % (
+            time() - start))
         pf = lambdify(var, f, 'math')
         psyf = None
         psyco = import_module('psyco')
@@ -542,50 +571,51 @@ def benchmark():
             t3 = Timer(code, 'from __main__ import psyf as f')
         else:
             t3 = None
-        print 'for x = (0, 1, 2, ..., 999)/1000'
-        print '20 times in 3 runs'
-        print 'compiled:      %.4f %.4f %.4f' % tuple(t1.repeat(3, 20))
-        print 'Python lambda: %.4f %.4f %.4f' % tuple(t2.repeat(3, 20))
+        print('for x = (0, 1, 2, ..., 999)/1000')
+        print('20 times in 3 runs')
+        print('compiled:      %.4f %.4f %.4f' % tuple(t1.repeat(3, 20)))
+        print('Python lambda: %.4f %.4f %.4f' % tuple(t2.repeat(3, 20)))
         if t3:
-            print 'Psyco lambda:  %.4f %.4f %.4f' % tuple(t3.repeat(3, 20))
+            print('Psyco lambda:  %.4f %.4f %.4f' % tuple(t3.repeat(3, 20)))
 
-    print 'big function:'
-    from sympy import diff, _exp, _sin, _cos, pi, lambdify
+    print('big function:')
+    from sympy import _exp, _sin, _cos, pi, lambdify
     x = Symbol('x')
 ##    f1 = diff(_exp(x)**2 - _sin(x)**pi, x) \
 ##        * x**12-2*x**3+2*_exp(x**2)-3*x**7+4*_exp(123+x-x**5+2*x**4) \
 ##        * ((x + pi)**5).expand()
     f1 = 2*_exp(x**2) + x**12*(-pi*_sin(x)**((-1) + pi)*_cos(x) + 2*_exp(2*x)) \
-         + 4*(10*pi**3*x**2 + 10*pi**2*x**3 + 5*pi*x**4 + 5*x*pi**4 + pi**5 \
-         + x**5)*_exp(123 + x + 2*x**4 - x**5) - 2*x**3 - 3*x**7
+        + 4*(10*pi**3*x**2 + 10*pi**2*x**3 + 5*pi*x**4 + 5*x*pi**4 + pi**5
+        + x**5)*_exp(123 + x + 2*x**4 - x**5) - 2*x**3 - 3*x**7
     fbenchmark(f1)
-    print
-    print 'simple function:'
+    print()
+    print('simple function:')
     y = Symbol('y')
-    f2 = sqrt(x*y)+x*5
-    fbenchmark(f2, [x,y])
+    f2 = sqrt(x*y) + x*5
+    fbenchmark(f2, [x, y])
     times = 100000
     fstr = '_exp(_sin(_exp(-x**2)) + sqrt(pi)*_cos(x**5/(x**3-x**2+pi*x)))'
     print
-    print 'frange with f(x) ='
-    print fstr
-    print 'for x=1, ..., %i' % times
-    print 'in 3 runs including full compile time'
+    print('frange with f(x) =')
+    print(fstr)
+    print('for x=1, ..., %i' % times)
+    print('in 3 runs including full compile time')
     t4 = Timer("frange('lambda x: %s', 0, %i)" % (fstr, times),
                'from __main__ import frange')
 
     numpy = import_module('numpy')
 
-    print 'frange:        %.4f %.4f %.4f' % tuple(t4.repeat(3, 1))
+    print('frange:        %.4f %.4f %.4f' % tuple(t4.repeat(3, 1)))
     if numpy:
         t5 = Timer('x = arange(%i); result = %s' % (times, fstr),
                    'from numpy import arange, sqrt, exp, sin, cos, exp, pi')
-        print 'numpy:         %.4f %.4f %.4f' % tuple(t5.repeat(3, 1))
+        print('numpy:         %.4f %.4f %.4f' % tuple(t5.repeat(3, 1)))
     # TODO: integration into fbenchmark
 
 if __name__ == '__main__':
     if __debug__:
-        print 'Running tests...',
+        print('Running tests...',)
+        numpy = import_module('numpy')
         test_cexpr()
         test_clambdify()
         test_frange()
@@ -595,7 +625,7 @@ if __name__ == '__main__':
         test_use_cse()
         import doctest
         doctest.testmod()
-        print 'OK'
+        print('OK')
         print
-    print 'Running benchmark...'
+    print('Running benchmark...')
     benchmark()
